@@ -28,47 +28,56 @@
 // aws ec2 describe-vpn-gateways --filters 'Name=attachment.vpc-id,Values='$vpc | grep VpnGatewayId
 // aws ec2 describe-network-interfaces --filters 'Name=vpc-id,Values='$vpc | grep NetworkInterfaceId
 
+use std::env;
+
 use aws_config::meta::region::RegionProviderChain;
 use aws_sdk_ec2 as ec2;
-use structopt::StructOpt;
+use clap::{Parser, Subcommand};
 
 use show::Show;
 
 mod aws;
 mod show;
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, Parser)]
 struct Aware {
-    #[structopt(
+    #[clap(
         help = "Explore resources from this region / these regions",
         long,
         short,
         global = true
     )]
     region: Vec<String>,
-    #[structopt(subcommand)]
+    #[clap(subcommand)]
     service: AwsService,
 }
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, Subcommand)]
 pub(crate) enum AwsService {
-    #[structopt(name = "ec2", about = "Explore EC2 resources")]
+    #[clap(name = "ec2", about = "Explore EC2 resources")]
     Ec2 {
-        #[structopt(long, short)]
+        #[clap(long, short)]
         vpc: Vec<String>,
     },
-    #[structopt(name = "cf", about = "Explore CloudFormation resources")]
+    #[clap(name = "cf", about = "Explore CloudFormation resources")]
     CloudFormation {
-        #[structopt(help = "Filter by given stack name", long)]
+        #[clap(help = "Filter by given stack name", long)]
         stack: Vec<String>,
-        #[structopt(help = "Filter by given stack status", long)]
+        #[clap(help = "Filter by given stack status", long)]
         status: Vec<aws::cf::model::StackStatus>,
     },
 }
 
 #[tokio::main]
 async fn main() -> anyhow::Result<()> {
-    let aware = Aware::from_args();
+    // Turn logging off by default
+    if env::var("RUST_LOG").is_err() {
+        env::set_var("RUST_LOG", "off");
+    }
+
+    tracing_subscriber::fmt::init();
+
+    let aware = Aware::parse();
 
     let regions = if aware.region.is_empty() {
         get_all_regions().await?
@@ -112,22 +121,19 @@ async fn collect_ec2(regions: Vec<String>, vpc: Vec<String>) -> anyhow::Result<(
             ),
         );
         progress.set_prefix(region.clone());
-        let mut aws = aws::Ec2Resources::new(client);
+        let mut ec2 = aws::Ec2Resources::new(client);
         progress.set_message("Collecting VPCs");
-        aws.collect_vpcs(&vpc).await?;
+        ec2.collect_vpcs(&vpc).await?;
         progress.inc(1);
 
-        aws.collect(&progress).await?;
+        ec2.collect(&progress).await?;
 
         progress.finish();
 
-        aws.vpcs()
-            .iter()
-            .map(|vpc| vpc_tree(&aws, vpc))
-            .for_each(|tree| {
-                println!();
-                ptree::print_tree(&tree).expect("Failed to print tree");
-            });
+        ec2.trees().for_each(|tree| {
+            println!();
+            ptree::print_tree(&tree).expect("Failed to print tree");
+        });
     }
 
     Ok(())
@@ -171,32 +177,3 @@ async fn collect_cf(
 
     Ok(())
 }
-
-fn vpc_tree(aws: &aws::Ec2Resources, vpc: &ec2::model::Vpc) -> ptree::item::StringItem {
-    let mut tree = ptree::TreeBuilder::new(vpc.id_and_name());
-    let tree = &mut tree;
-    let vpc_id = vpc.id();
-    add_children(tree, "Subnets", aws.subnets(&vpc_id));
-    add_children(tree, "Instances", aws.instances(&vpc_id));
-    add_children(tree, "Internet Gateways", aws.internet_gateways(&vpc_id));
-    add_children(tree, "Route Tables", aws.route_tables(&vpc_id));
-    add_children(tree, "Network ACLs", aws.network_acls(&vpc_id));
-    add_children(tree, "VPC Peering Connections", aws.vpc_peerings(&vpc_id));
-    add_children(tree, "VPC Endpoints", aws.vpc_endpoints(&vpc_id));
-    add_children(tree, "NAT Gateways", aws.nat_gateways(&vpc_id));
-    add_children(tree, "Security Groups", aws.security_groups(&vpc_id));
-    add_children(tree, "VPN Connections", aws.vpn_connections(&vpc_id));
-    add_children(tree, "VPN Gateways", aws.vpn_gateways(&vpc_id));
-    add_children(tree, "Network Interfaces", aws.network_interfaces(&vpc_id));
-    tree.build()
-}
-
-fn add_children(ptree: &mut ptree::TreeBuilder, title: impl ToString, resources: Vec<impl Show>) {
-    if !resources.is_empty() {
-        ptree.begin_child(title.to_string());
-        resources.into_iter().for_each(|resource| {
-            ptree.add_empty_child(resource.id_and_name());
-        });
-        ptree.end_child();
-    }
-}
